<?php

namespace Drupal\entrasync\Entity\ListBuilder;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;

/**
 * Defines a class to build a listing of sync entities.
 */
class SyncEntityListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('Name');
    $header['info'] = $this->t('Information');
    $header['operations'] = $this->t('Operations');
    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {

    $row['id'] = [
      'data' => [
        '#type' => 'markup',
        '#markup' => '<strong>' . $entity->label() . '</strong><br />' .
                      '<small>id: <em>' . $entity->id() . '</em></small>',
        '#allowed_tags' => ['strong', 'br', 'small'],
      ],
    ];

    // Get the key information for the help text.
    $used_key = \Drupal::service('key.repository')->getKey($entity->get('graph_key'));
    $key_info = $this->t('<strong>Key: </strong>@key_label<br />', [ '@key_label' => $used_key->label() ]);

    // Get the filter information for the help text.
    $has_filter = $entity->get('filter');
    $filter_property = $entity->get('entrafilter_property');
    $filter_operator = $entity->get('entrafilter_operator');
    $operator_mapping = [
      'equals' => $this->t('equals'),
      'notequals' => $this->t('does not equal'),
      'startswith' => $this->t('starts with'),
      'endsWith' => $this->t('ends with'),
      'contains' => $this->t('contains'),
    ];
    $human_readable_operator = $operator_mapping[$filter_operator] ?? $filter_operator;
    $filter_value = $entity->get('entrafilter_value');

    $filter_info = '';
    if (!empty($has_filter)) {
      $filter_info = $this->t('<strong>Filter: </strong>@filter_property <small> that </small> @filter_operator "@filter_value"<br />', [
        '@filter_property' => $filter_property,
        '@filter_operator' => $human_readable_operator,
        '@filter_value' => $filter_value,
      ]);
    }

    // Get the mapping information for the help text.
    $mapped_to_entity = $entity->get('entrauser_entities');
    $num_fields_mapped = count($entity->get('user_field_mapping'));
    $mapping_info = $this->t('<strong>Mapping: </strong>@num_fields fields mapped to @mapped_to_entity entity', [
      '@num_fields' => $num_fields_mapped,
      '@mapped_to_entity' => $mapped_to_entity,
    ]);

    // Build the actual help text:
    $row['info'] = [
      'data' => [
        '#type' => 'markup',
        '#markup' => $key_info . $filter_info . $mapping_info,
        '#allowed_tags' => ['strong', 'br', 'small'],
      ],
    ];

    // Get the default operations.
    $operations = $this->buildOperations($entity)['#links'];

    // Add custom operations.
    $operations['perform_sync'] = [
      'title' => $this->t('Perform Sync'),
      'url' => $entity->toUrl('sync'),
      'weight' => 10,
    ];
    $operations['update_sync_status'] = [
      'title' => $this->t('Update Status'),
      'url' => $entity->toUrl('update-status'),
      'weight' => 15,
    ];

    $row['operations']['data'] = [
      '#type' => 'operations',
      '#links' => $operations,
    ];

    return $row;
  }

}
